@@ -1,4 +1,3 @@
-from typing import List
 import logging
 from .vcenter import VcenterConfig
 from .vmware_guest_facts import vmware_guest_facts
@@ -9,59 +8,76 @@ from .vmware_resource_pool_facts import vmware_resource_pool_facts
 from .vmware_portgroup_facts import vmware_portgroup_facts
 from .vmware_vm_facts import vmware_vm_facts
 from .vmware_vswitch_facts import vmware_vswitch_facts
+from typing import List
+import concurrent.futures
 
 logger = logging.getLogger(__name__)
 
 
 class VMWARE:
 
-    def __init__(self,VcenterConfig):
+    def __init__(self, VcenterConfig):
         self.VcenterConfig = VcenterConfig
 
+    def vmware_vswitch_facts(self, esxi_hostname):
+        return vmware_vswitch_facts(self.VcenterConfig, esxi_hostname)
 
-    def vmware_vswitch_facts(self,esxi_hostname):
-        return vmware_vswitch_facts(self.VcenterConfig,esxi_hostname)
+    def vmware_guest_facts(self, show_attribute=False, datacenter='Datacenter', uuid='', schema='summary', name=''):
+        return vmware_guest_facts(self.VcenterConfig, show_attribute, datacenter, uuid, schema, name)
 
-    def vmware_guest_facts(self,show_attribute=False,datacenter='Datacenter',uuid='',schema='summary',name=''):
-        return vmware_guest_facts(self.VcenterConfig,show_attribute,datacenter,uuid,schema,name)
+    def vmware_datastore_facts(self, name):
+        return vmware_datastore_facts(self.VcenterConfig, name)
 
-    def vmware_datastore_facts(self,name):
-        return vmware_datastore_facts(self.VcenterConfig,name)
+    def vmware_vm_facts(self, *args, **kwargs):
+        return vmware_vm_facts(self.VcenterConfig, *args, **kwargs)
 
-    def vmware_vm_facts(self,*args,**kwargs):
-        return vmware_vm_facts(self.VcenterConfig,*args,**kwargs)
-
-    def vmware_portgroup_facts(self,esxi_hostname):
-        return vmware_portgroup_facts(self.VcenterConfig,esxi_hostname)
+    def vmware_portgroup_facts(self, esxi_hostname):
+        return vmware_portgroup_facts(self.VcenterConfig, esxi_hostname)
 
     def vmware_resource_pool_facts(self):
         return vmware_resource_pool_facts(self.VcenterConfig)
 
-    def vmware_host_facts(self,esxi_hostname):
-        return vmware_host_facts(self.VcenterConfig,esxi_hostname)
+    def vmware_host_facts(self, esxi_hostname):
+        return vmware_host_facts(self.VcenterConfig, esxi_hostname)
 
-    def vmware_guest_disk_info(self,moid):
-        return vmware_guest_disk_info(self.VcenterConfig,moid)
+    def vmware_guest_disk_info(self, moid):
+        return vmware_guest_disk_info(self.VcenterConfig, moid)
 
 
 class Vsphere:
 
-    def __init__(self,VcenterConfig:VcenterConfig):
+    def __init__(self, VcenterConfig: VcenterConfig):
         self.vmware = VMWARE(VcenterConfig)
         self._datastores = {}
         self._vm = {}
         self._host = {}
-        self.get()
-        self.clean()
+        self._network = {}
+        self._cluster = {}
+        self.exclude_vm = {}
+        # self.get()
+        # self.clean()
 
 
+    @property
+    def cluster(self):
+        if not self._cluster:
+            return {}
+        else:
+            return self._cluster
 
-    def get_datastores(self,name=None) -> dict:
+    @property
+    def network(self):
+        if not self._network:
+            return {}
+        else:
+            return self._network
+
+    def get_datastores(self, name=None) -> dict:
         for datastore in self.vmware.vmware_datastore_facts(name):
             key = datastore['name']
             value = datastore
             self._datastores[key] = value
-            logging.info(f"{key} {value}")
+            logger.info(f"{key} {value}")
 
         return self._datastores
 
@@ -72,7 +88,7 @@ class Vsphere:
         else:
             return self._datastores
 
-    def get_host_detail(self,resource):
+    def get_host_detail(self, resource):
         resource.update(self.vmware.vmware_host_facts(esxi_hostname=resource['owner']))
         for datastore in resource['ansible_datastore']:
             datastore.update(self.vmware.vmware_datastore_facts(name=datastore['name']))
@@ -82,17 +98,25 @@ class Vsphere:
         return resource
 
     def get_hosts(self):
-        resource_objs = self.vmware.vmware_resource_pool_facts()
+        resource_objs = []
 
-        for resource in resource_objs:
-            key = resource['owner']
-            value = self.get_host_detail(resource)
-            self._host[key] = value
-            logging.info(f"{key} {value}")
+        for resource in self.vmware.vmware_resource_pool_facts():
+            # key = resource['owner']
+            if resource['runtime_memory_reservation_used'] != 0:
+                resource_objs.append(resource)
+
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.get_host_detail, resource) for resource in resource_objs]
+            for future in concurrent.futures.as_completed(futures):
+                try:
+                    result = future.result()
+                    self._host[result['owner']] = result
+                except Exception as exc:
+                    print('get_hosts generated an exception: %s' % (exc))
+                    logger.error('get_hosts generated an exception: %s' % (exc))
 
         return self._host
 
-
     @property
     def hosts(self):
         if not self._host:
@@ -100,23 +124,33 @@ class Vsphere:
         else:
             return self._host
 
-
     def get_vm(self):
-        vm_objs = self.vmware.vmware_vm_facts()
+        vm_objs = []
 
-        for vm in vm_objs:
+        for vm in self.vmware.vmware_vm_facts():
             if not vm['uuid']:
                 continue
-
             else:
+                vm_objs.append(vm)
+                # try:
+                #     self._vm[vm['guest_name']] = self.get_vm_datatil(vm)
+                # except Exception as e:
+                #     logger.error(f"{vm['guest_name']} ERROR {e}")
+
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            futures = [executor.submit(self.get_vm_datatil, vm) for vm in vm_objs]
+            for future in concurrent.futures.as_completed(futures):
                 try:
-                    self._vm[vm['guest_name']] = self.get_vm_datatil(vm)
-                except Exception as e:
-                    print(vm['guest_name'],'ERROR',e)
+                    result = future.result()
+                    self._vm[result['hw_name']] = result
+                except Exception as exc:
+                    print('vm_objs generated an exception: %s' % (exc))
+                    # logger.error(f"{vm['guest_name']} ERROR {e}")
+
+
         return self._vm
 
-
-    def get_vm_datatil(self,vm):
+    def get_vm_datatil(self, vm):
         data = self.vmware.vmware_guest_facts(uuid=vm['uuid'])
         capacity_in_kb = sum([v['capacity_in_kb'] for v in self.vmware.vmware_guest_disk_info(moid=data['moid']).values()])
         capacity = capacity_in_kb / 1024 / 1024
@@ -136,11 +170,16 @@ class Vsphere:
         self.get_hosts()
         self.get_vm()
 
-    def clean_host(self,hostname):
+    def clean_host(self, hostname):
         data = self._host[hostname]
 
         # models.NetWork 跟著集群
-        network_objs = []
+
+
+        data['cluster'] = data['cluster'] if data['cluster'] else data['ansible_hostname']
+        self._cluster[data['cluster']] = {'name': data['cluster'], 'network': data['network']}
+
+
         for network in data['network']:
             network.update(
                 {
@@ -148,24 +187,16 @@ class Vsphere:
                     'status': True,
                     'dhcp': True
                 })
-            network_objs.append(network)
+            self._network[network['network']] = network
+            # data['network'].append(network['network'])
+            logger.info(f"network {network['network']} {network}")
 
-            print('network_objs %s' % network_objs)
-
-        if data['cluster']:
-            cluster = data['cluster']
-        else:
-            # 如果沒有cluster 那麼自己就是cluster
-            cluster = data['ansible_hostname']
-
-        data['cluster'] = cluster
 
         datastore_objs = []
         for datastore in data['ansible_datastore']:
             datastore_obj = self._datastores[datastore['name']]
             datastore_objs.append(datastore_obj)
 
-
         # models.Host
         data['name'] = data['ansible_hostname']
         data['ansible_all_ipv4_addresses'] = data['ansible_all_ipv4_addresses'][0] if data[
@@ -175,7 +206,7 @@ class Vsphere:
         self._host[hostname].update(data)
         return data
 
-    def clean_vm(self,name):
+    def clean_vm(self, name):
         data = self._vm[name]
         hostname = data['hw_esxi_host']
         host = self.hosts[hostname]
@@ -184,8 +215,7 @@ class Vsphere:
         datastore = self._datastores[datastore]
 
         summarys = [v['summary'] for k, v in data.items() if 'eth' in k]
-        network_obj = None
-
+        network = [i for i in summarys if self._network.get(i)][0] if [i for i in summarys if self._network.get(i)] else None
 
         mac_address = [x['macaddress'] for x in [v for k, v in data.items() if 'eth0' in k] if x['ipaddresses']]
         mac_address = mac_address[0] if mac_address else ''
@@ -201,26 +231,37 @@ class Vsphere:
             'capacity': int(data['capacity']),
             'instance_uuid': data['instance_uuid'],
             'datastore': datastore,
-            'network': network_obj,
+            'network': network,
             'hw_guest_full_name': data['hw_guest_full_name'] if data['hw_guest_full_name'] else '',
 
         })
 
         if data['hw_power_status'] == 'poweredOff':
             # 實例已停止 不更新IP
-            print("實例已停止 不更新IP")
+            logger.info("實例已停止 不更新IP")
 
         self._vm[name].update(data)
         return data
 
+    def clean_host_list(self):
+        for name in self.hosts.keys():
+            self.clean_host(name)
 
+    def clean_vm_list(self):
+        self.include_vm = {k: v for k, v in self.vms.items() if v['hw_guest_id']}
+        self.exclude_vm = {k: v for k, v in self.vms.items() if not v['hw_guest_id']}
+
+        for name in self.include_vm.keys():
+            self.clean_vm(name)
 
     def clean(self):
-        for name in self.vms.keys():
-            self.clean_vm(name)
+        self.clean_host_list()
+        self.clean_vm_list()
+
+
 
 
 
 if __name__ == '__main__':
-    vcenterconfig = VcenterConfig(hostname='192.168.10.1',username='abc',password='123456')
+    vcenterconfig = VcenterConfig(hostname='192.168.10.1', username='abc', password='123456')
     vsphere = Vsphere(vcenterconfig)
